"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const protobufjs = require("protobufjs");
const path = require("path");
/**
 * @version 1.0 protobuff 协议管理器
 */
class ProtoBuffService {
    constructor() {
        this.urlMapping = new Map();
        this.rootMapping = new Map();
        //默认加载router.proto
        this.rootPath = path.join(__dirname, "../../protobuff", "router.proto");
        this.routeRoot = this.addProtoRoot(this.rootPath);
        //取出类型
        let service = this.routeRoot.lookupService("Router");
        let method = service.methods["transferRoute"];
        this.routeHandleType = this.routeRoot.lookupType(method.requestType);
    }
    getRouteRoot() {
        return this.routeRoot;
    }
    getRouterRootPath() {
        return this.rootPath;
    }
    addProtoRoot(fp) {
        let content = this.rootMapping.get(fp);
        if (!!content) {
            return content.root;
        }
        const root = protobufjs.loadSync(fp);
        this.rootMapping.set(fp, {
            protoPath: fp,
            root,
        });
        return root;
    }
    addUrlMapping(p) {
        if (this.urlMapping.has(p.url)) {
            return true;
        }
        this.addProtoRoot(p.protoPath);
        this.urlMapping.set(p.url, p);
        return true;
    }
    //根据url进行压缩  type 向上true还是向下false
    encode(msg, url, type) {
        let urlConfig = this.urlMapping.get(url);
        if (!!urlConfig) {
            let rootM = this.rootMapping.get(urlConfig.protoPath);
            if (rootM) {
                let root = rootM.root;
                let service = root.lookupService(urlConfig.service);
                let method = service.methods[urlConfig.method];
                let reType = type ? method.requestType : method.responseType;
                const HandleMessage = root.lookupType(reType);
                let message = HandleMessage.create(msg);
                let buffer = HandleMessage.encode(message).finish();
                return Array.from(buffer);
            }
        }
        return Array.from(Buffer.from(JSON.stringify(msg)));
    }
    decode(buffer, url, type) {
        let urlConfig = this.urlMapping.get(url);
        if (!!urlConfig) {
            let rootM = this.rootMapping.get(urlConfig.protoPath);
            if (rootM) {
                let root = rootM.root;
                let service = root.lookupService(urlConfig.service);
                let method = service.methods[urlConfig.method];
                let reType = type ? method.requestType : method.responseType;
                const HandleMessage = root.lookupType(reType);
                let bytes = Buffer.from(buffer);
                let message = HandleMessage.decode(bytes).toJSON();
                return message;
            }
        }
        if (Array.isArray(buffer) || typeof buffer == "string") {
            let finalMsg = Buffer.from(buffer).toString();
            try {
                let obj = JSON.parse(finalMsg);
                if (!!obj && typeof obj === "object") {
                    return obj;
                }
            }
            catch (e) { }
        }
        return buffer;
    }
    //路由压缩
    encodeRoute(msg) {
        let message = this.routeHandleType.create(msg);
        return this.routeHandleType.encode(message).finish();
    }
    //路由消息解压
    decodeRoute(buffer) {
        return this.routeHandleType.decode(buffer).toJSON();
    }
}
exports.default = new ProtoBuffService();
