"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const fastcar_core_1 = require("fastcar-core");
const RpcConfig_1 = require("../../types/RpcConfig");
const SocketFactory_1 = require("../socket/SocketFactory");
const fastcar_timer_1 = require("fastcar-timer");
const TaskAsync_1 = require("../../model/TaskAsync");
const utils_1 = require("fastcar-core/utils");
const RpcUrlData_1 = require("../../constant/RpcUrlData");
const annotation_1 = require("fastcar-core/annotation");
const ProtoBuffService_1 = require("../ProtoBuffService");
//封装一个可用的rpc框架
let RpcClient = class RpcClient {
    constructor(config, rpcAsyncService, retry) {
        let ClientClass = (0, SocketFactory_1.SocketClientFactory)(config.type);
        if (!ClientClass) {
            this.rpcLogger.error(`Failed to create this client type by ${config.type}`);
            throw new Error(`Failed to create this client type by ${config.type}`);
        }
        this.config = Object.assign({
            retryCount: 3,
            retryInterval: 100,
            maxMsgNum: 10000,
            timeout: 3000,
            disconnectInterval: 100,
        }, config, retry);
        this.client = new ClientClass(this.config, this);
        this.msgQueue = new Map();
        this.serialId = 0;
        this.checkStatus = false;
        this.rpcAsyncService = rpcAsyncService;
        this.checkConnectTimer = 0;
    }
    //初始化配置事件
    addProtoBuf(p) {
        //先加载root节点
        let root = ProtoBuffService_1.default.addProtoRoot(p.root.protoPath);
        if (p.prefixUrl && !p.prefixUrl.startsWith("/")) {
            p.prefixUrl = `/${p.prefixUrl}`;
        }
        let service = root.lookupService(p.root.service);
        let methods = Object.keys(service.methods);
        let pMap = new Map();
        if (p?.list && p.list.length > 0) {
            p.list.forEach((item) => {
                pMap.set(item.method, item.url);
            });
        }
        methods.forEach((m) => {
            let pp = {
                protoPath: p.root.protoPath,
                service: p.root.service,
                url: pMap.has(m) ? pMap.get(m) : `/${m}`,
                method: m,
            };
            if (!pp.url.startsWith("/")) {
                pp.url = "/" + pp.url;
            }
            if (p.prefixUrl) {
                if (!pp.url.startsWith(p.prefixUrl)) {
                    pp.url = p.prefixUrl + pp.url;
                }
            }
            ProtoBuffService_1.default.addUrlMapping(pp);
        });
    }
    addSerialId() {
        if (this.msgQueue.size >= this.config.maxMsgNum) {
            return -1;
        }
        for (let i = 1; i < this.config.maxMsgNum; i++) {
            if (this.serialId >= this.config.maxMsgNum) {
                this.serialId = 1;
            }
            else {
                this.serialId++;
            }
            if (!this.msgQueue.has(this.serialId)) {
                return this.serialId;
            }
        }
        return -1;
    }
    async start() {
        return new Promise((resolve) => {
            this.client.connect();
            setTimeout(() => {
                resolve(this.client.connected);
            }, 50);
        });
    }
    stop(reason) {
        this.client.offline(reason);
    }
    close() {
        this.checkMsg(0, true);
        this.client.close();
    }
    getClient() {
        return this.client;
    }
    //是否已经连接
    isConnect() {
        return this.client.connected;
    }
    isForceConnect() {
        return this.client.forceConnect;
    }
    getSessionId() {
        return this.client.sessionId;
    }
    async handleMsg(msg) {
        if (RpcConfig_1.InteractiveMode.response == msg.mode) {
            let id = msg.id;
            if (!!id) {
                let item = this.msgQueue.get(id);
                if (item) {
                    item.cb.done(null, msg);
                }
                this.msgQueue.delete(id);
            }
        }
        else {
            if (RpcConfig_1.InteractiveMode.notify == msg.mode) {
                if (msg.url == RpcUrlData_1.RpcUrlData.forceDisconnect) {
                    this.client.forceConnect = true;
                    let data = msg.data;
                    let reason = data?.reason;
                    this.client.offline(reason);
                }
                else {
                    this.rpcAsyncService.handleMsg(msg.url, msg.data || {});
                }
            }
            else {
                let repData = await this.rpcAsyncService.handleMsg(msg.url, msg.data || {});
                this.client.sendMsg({
                    id: msg.id,
                    data: repData || {},
                    url: msg.url,
                    mode: RpcConfig_1.InteractiveMode.response,
                });
            }
        }
    }
    //发送消息
    async request(url, data, opts) {
        let id = this.addSerialId();
        let m = Object.assign({
            url,
            data: data,
        }, {
            retryCount: this.config.retryCount,
            retryInterval: this.config.retryInterval,
            timeout: this.config.timeout, //超时时间
        }, {
            retryCount: opts?.retryCount,
            retryInterval: opts?.retryInterval,
            timeout: opts?.timeout, //超时时间
        });
        return new Promise((resolve) => {
            // if (!this.client.connected) {
            // 	resolve({ code: RpcResponseCode.disconnect, msg: "socket is disconnect" });
            // 	return;
            // }
            if (id == -1) {
                //消息太多没有处理完
                resolve({ code: RpcConfig_1.RpcResponseCode.busy, msg: "The message number has been used up" });
                return;
            }
            let timeout = m.timeout || this.config.timeout;
            let msg = {
                id,
                url: m.url,
                mode: RpcConfig_1.InteractiveMode.request,
            };
            if (utils_1.ValidationUtil.isNotNull(m.data)) {
                msg.data = m.data;
            }
            let cb = new TaskAsync_1.default((err, res) => {
                err ? resolve(err) : resolve({ code: RpcConfig_1.RpcResponseCode.ok, data: res.data } || { code: RpcConfig_1.RpcResponseCode.ok });
            });
            let rpcMsg = {
                id,
                msg,
                retryCount: 0,
                retryInterval: m.retryInterval,
                expiretime: Date.now() + timeout,
                timeout,
                maxRetryCount: m.retryCount,
                maxRetryInterval: m.retryInterval,
                cb,
            };
            this.msgQueue.set(id, rpcMsg);
            this.client.sendMsg(msg);
        });
    }
    async checkMsg(diff, stop) {
        if (this.checkStatus) {
            this.rpcLogger.warn("Client message detection time is too long");
            return;
        }
        try {
            this.checkStatus = true;
            let nowTime = Date.now();
            if (this.msgQueue.size > 0) {
                let cleanIds = new Map();
                this.msgQueue.forEach((item, id) => {
                    if (nowTime > item.expiretime) {
                        cleanIds.set(id, {
                            code: RpcConfig_1.RpcResponseCode.timeout,
                            msg: `The message times out ${item.timeout} ms`,
                        });
                        return;
                    }
                    //如果是断线的则进行堆积
                    if (!this.isConnect()) {
                        return;
                    }
                    item.retryInterval -= diff;
                    if (item.retryInterval <= 0) {
                        //发送消息
                        if (item.retryCount >= item.maxRetryCount) {
                            cleanIds.set(id, {
                                code: RpcConfig_1.RpcResponseCode.retryTimes,
                                msg: `The message retries more than ${item.retryCount} times`,
                            });
                        }
                        else {
                            //重发消息
                            item.retryCount++;
                            this.client.sendMsg(item.msg);
                        }
                        item.retryInterval = item.maxRetryInterval;
                    }
                });
                cleanIds.forEach((resp, id) => {
                    let item = this.msgQueue.get(id);
                    if (item) {
                        this.rpcLogger.error(resp);
                        this.rpcLogger.error(`${JSON.stringify(item.msg)}`);
                        item.cb.done(resp, null);
                    }
                    this.msgQueue.delete(id);
                });
            }
        }
        catch (e) {
            this.rpcLogger.error("rpc client checkMsgQueue error");
            this.rpcLogger.error(e);
        }
        finally {
            this.checkStatus = false;
        }
    }
};
__decorate([
    (0, annotation_1.Log)("rpc-client"),
    __metadata("design:type", fastcar_core_1.Logger)
], RpcClient.prototype, "rpcLogger", void 0);
__decorate([
    (0, fastcar_timer_1.ScheduledInterval)({
        initialDelay: 1000,
        fixedRate: 100, //大致100ms去check一次
    }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Boolean]),
    __metadata("design:returntype", Promise)
], RpcClient.prototype, "checkMsg", null);
RpcClient = __decorate([
    fastcar_timer_1.EnableScheduling,
    __metadata("design:paramtypes", [Object, Object, Object])
], RpcClient);
exports.default = RpcClient;
