"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const fastcar_core_1 = require("fastcar-core");
const annotation_1 = require("fastcar-core/annotation");
const RpcConfig_1 = require("../../types/RpcConfig");
const ComposeService_1 = require("../ComposeService");
const SocketManager_1 = require("../socket/SocketManager");
const TaskAsync_1 = require("../../model/TaskAsync");
const fastcar_timer_1 = require("fastcar-timer");
const RpcMetaData_1 = require("../../constant/RpcMetaData");
const RpcUrlData_1 = require("../../constant/RpcUrlData");
const SocketMsgStatus_1 = require("../../constant/SocketMsgStatus");
const ProtoBuffService_1 = require("../ProtoBuffService");
//rpc 管理服务 用于和客户端进行同步异步消息发送
let RpcServer = class RpcServer {
    constructor() {
        this.middleware = [];
        this.rcpRouterMap = new Map();
        this.socketManager = new SocketManager_1.default();
        this.msgQueue = new Map();
        this.serialId = 0;
        this.rpcConfig = {
            list: [],
            retry: {
                retryCount: 3,
                retryInterval: 100,
                maxMsgNum: 10000,
                timeout: 3000,
            },
        };
        this.failMsgQueue = [];
        this.checkStatus = false;
    }
    //序列号递增
    addSerialId() {
        let rpcRetryPolicyConfig = this.rpcConfig.retry;
        if (this.msgQueue.size >= rpcRetryPolicyConfig.maxMsgNum) {
            return -1;
        }
        for (let i = 1; i < rpcRetryPolicyConfig.maxMsgNum; i++) {
            if (this.serialId >= rpcRetryPolicyConfig.maxMsgNum) {
                this.serialId = 1;
            }
            else {
                this.serialId++;
            }
            if (!this.msgQueue.has(this.serialId)) {
                return this.serialId;
            }
        }
        return -1;
    }
    //封装请求 做出回应
    response() {
        return async (context, next) => {
            let result = {
                sessionId: context.sessionId,
                msg: { id: context.id, url: context.url, mode: RpcConfig_1.InteractiveMode.response },
            };
            try {
                if (next) {
                    await next();
                }
                let res = context.body;
                if (!res) {
                    result.msg.data = {
                        code: RpcConfig_1.RpcResponseCode.notfound,
                        msg: "not found",
                    };
                }
                else {
                    result.msg.data = res;
                }
            }
            catch (e) {
                this.rpcLogger.error(`${context.url} is error`);
                this.rpcLogger.error(e);
                result.msg.data = {
                    code: RpcConfig_1.RpcResponseCode.error,
                    msg: "Server internal error",
                };
            }
            finally {
                if (context.url == RpcUrlData_1.RpcUrlData.disconnect) {
                    //断线了就不发消息了
                    return;
                }
                this.sendMsgBySessionId(result);
            }
        };
    }
    /***
     * @version 1.0 加载路由
     *
     */
    loadRoute() {
        let instanceList = this.app.getComponentByType(fastcar_core_1.ComponentKind.Controller);
        //查找绑定的url
        instanceList.forEach((instance) => {
            let routerMap = Reflect.getMetadata(RpcMetaData_1.RpcMetaData.RPCMethod, instance);
            if (!routerMap || routerMap.size == 0) {
                return;
            }
            //绑定具体的protobuff协议
            let protoMeta = Reflect.getMetadata(RpcMetaData_1.RpcMetaData.ProtoDataConfig, instance);
            routerMap.forEach((item, url) => {
                const callBack = async (ctx) => {
                    let data = ctx.data || {};
                    let res = await instance[item.method](data, ctx);
                    if (!!res) {
                        ctx.body = res;
                    }
                };
                this.rcpRouterMap.set(url, callBack);
                if (!!protoMeta) {
                    ProtoBuffService_1.default.addUrlMapping({
                        url,
                        protoPath: protoMeta.protoPath,
                        service: protoMeta.service,
                        method: item.method,
                    });
                }
            });
        });
        if (this.rcpRouterMap.size == 0) {
            return null;
        }
        //构造一个中间件查找路由
        return async (ctx, next) => {
            if (!ctx.url) {
                if (next) {
                    next();
                }
                return;
            }
            let cb = this.rcpRouterMap.get(ctx.url);
            if (!cb) {
                if (next) {
                    next();
                }
                return;
            }
            await Promise.resolve(Reflect.apply(cb, this, [ctx]));
            if (next) {
                next();
            }
        };
    }
    connect(session) {
        //传递调用
        this.handleMsg(session, {
            url: RpcUrlData_1.RpcUrlData.connect,
            data: session,
            mode: RpcConfig_1.InteractiveMode.request,
        });
    }
    async auth(username, password, session) {
        let config = this.socketManager.getSocketServerConfig(session.serverId);
        if (!config) {
            return false;
        }
        let service = this.app.getComponentByName(RpcMetaData_1.RpcMetaData.RPCAuthService);
        if (!service) {
            if (!config.secure?.password || !config.secure.username) {
                return true;
            }
            return config.secure.password === password && config.secure.username === username;
        }
        return await service.auth(username, password, config);
    }
    disconnect(session, reason) {
        this.handleMsg(session, {
            url: RpcUrlData_1.RpcUrlData.disconnect,
            data: { session, reason },
            mode: RpcConfig_1.InteractiveMode.request,
        });
    }
    handleMsg(session, msg) {
        //如果为应答模式
        if (msg.mode == RpcConfig_1.InteractiveMode.response) {
            if (msg.id) {
                let item = this.msgQueue.get(msg.id);
                if (item) {
                    let cb = item.cb;
                    if (cb) {
                        cb.done(null, msg);
                        this.msgQueue.delete(msg.id);
                    }
                }
            }
            return;
        }
        let ctx = Object.assign({}, msg, session);
        //调用绑定的组件
        this.composeMiddleware(ctx);
    }
    getSocketManager() {
        return this.socketManager;
    }
    //给单个会话发送消息
    async sendMsgBySessionId(m) {
        let flag = await this.socketManager.sendMsg(m.sessionId, m.msg);
        if (flag == SocketMsgStatus_1.SocketMsgStatus.fail) {
            let retry = this.rpcConfig.retry;
            let timeout = m.timeout || retry.timeout;
            //放入至失败的消息队列中进行处理
            this.pushFailMsg({
                sessionId: m.sessionId,
                msg: m.msg,
                retryCount: 0,
                retryInterval: m.retryInterval || retry.retryInterval,
                expiretime: Date.now() + timeout,
                timeout: timeout,
                maxRetryCount: m.retryCount || retry.retryCount,
                maxRetryInterval: m.retryInterval || retry.retryInterval,
            });
        }
    }
    //重试消息
    async retrySendMsg(m) {
        let flag = await this.socketManager.sendMsg(m.sessionId, m.msg);
        return flag;
    }
    pushFailMsg(m) {
        if (this.failMsgQueue.length >= this.rpcConfig.retry.maxMsgNum) {
            //将之前的抛出
            let item = this.failMsgQueue.shift();
            if (item) {
                this.rpcLogger.error("The message queue is too long and is discarded");
                this.rpcLogger.error(item.msg);
            }
        }
        this.failMsgQueue.push(m);
    }
    /***
     * @version 1.0 异步通知消息 不保证客户端一定能收到消息
     *
     */
    notifyMessage(msg) {
        if (msg.channel) {
            this.socketManager.sendMsgByChannel(msg.channel, {
                url: msg.url,
                data: msg.data,
                mode: RpcConfig_1.InteractiveMode.notify,
            }, msg.excludeIds);
        }
        else if (msg.sessionId) {
            this.sendMsgBySessionId({
                sessionId: msg.sessionId,
                msg: {
                    url: msg.url,
                    data: msg.data,
                    mode: RpcConfig_1.InteractiveMode.notify,
                },
            });
        }
    }
    /***
     * @version 1.0 向客户端发起请求
     * @params  sessionId 会话id
     * @params  data 发起的数据
     * @params timeout 超时后返回 默认3s
     *
     */
    async request(sessionId, url, data, opts) {
        return new Promise((resolve) => {
            if (!this.socketManager.sessionOnline(sessionId)) {
                resolve({ code: RpcConfig_1.RpcResponseCode.disconnect, msg: "socket is disconnect" });
                return;
            }
            let id = this.addSerialId();
            if (id == -1) {
                resolve({ code: RpcConfig_1.RpcResponseCode.busy, msg: "The message number has been used up" });
                return;
            }
            let timeout = opts?.timeout || this.rpcConfig.retry.timeout;
            let r = {
                sessionId,
                msg: {
                    id,
                    url,
                    data,
                    mode: RpcConfig_1.InteractiveMode.request,
                },
                retryCount: opts?.retryCount || this.rpcConfig.retry.retryCount,
                retryInterval: opts?.retryInterval || this.rpcConfig.retry.retryInterval,
                timeout: timeout,
            };
            let cb = new TaskAsync_1.default((err, res) => {
                err ? resolve(err) : resolve({ code: RpcConfig_1.RpcResponseCode.ok, data: res.data });
            });
            let m = {
                id,
                cb,
                timeout: timeout,
                expiretime: Date.now() + timeout,
            };
            this.msgQueue.set(id, m);
            this.sendMsgBySessionId(r);
        });
    }
    //强制下线
    kickSessionId(sessionId, reason) {
        this.notifyMessage({
            url: RpcUrlData_1.RpcUrlData.forceDisconnect,
            data: { reason },
            sessionId: sessionId,
        });
        queueMicrotask(() => {
            //向客户端通知强制掉线
            this.socketManager.disconnect(sessionId, reason, true);
        });
    }
    //失败的消息处理
    handleFailMsg(msg, res) {
        this.rpcLogger.error(res.msg);
        this.rpcLogger.error(msg);
        if (msg.id) {
            let msgItem = this.msgQueue.get(msg.id);
            if (msgItem) {
                msgItem.cb.done(res, null);
                this.msgQueue.delete(msg.id);
            }
        }
    }
    async checkMsgQueue(diff, stop) {
        if (this.checkStatus) {
            this.rpcLogger.warn("Client message detection time is too long");
            return;
        }
        try {
            let nowTime = Date.now();
            this.checkStatus = true;
            if (this.failMsgQueue.length > 0) {
                let newFailMsgQueue = [];
                for (let item of this.failMsgQueue) {
                    item.retryInterval -= diff;
                    if (item.retryInterval <= 0) {
                        //进行发送
                        let flag = await this.retrySendMsg(item);
                        if (flag == SocketMsgStatus_1.SocketMsgStatus.success) {
                            continue;
                        }
                        if (flag == SocketMsgStatus_1.SocketMsgStatus.offline) {
                            this.handleFailMsg(item.msg, {
                                code: RpcConfig_1.RpcResponseCode.disconnect,
                                msg: `session offline`,
                            });
                            continue;
                        }
                        item.retryInterval = item.maxRetryInterval;
                        item.retryCount++;
                        if (item.retryCount >= item.maxRetryCount) {
                            this.handleFailMsg(item.msg, {
                                code: RpcConfig_1.RpcResponseCode.retryTimes,
                                msg: `The message retries more than ${item.retryCount} times`,
                            });
                            continue;
                        }
                    }
                    if (nowTime > item.expiretime) {
                        this.handleFailMsg(item.msg, {
                            code: RpcConfig_1.RpcResponseCode.timeout,
                            msg: `The message times out ${item.timeout} ms`,
                        });
                        continue;
                    }
                    newFailMsgQueue.push(item);
                }
                this.failMsgQueue = newFailMsgQueue;
            }
            //检测发生过去消息但是依然没收到返回的情况
            let cleanIds = [];
            this.msgQueue.forEach((item, id) => {
                if (nowTime > item.expiretime) {
                    cleanIds.push(id);
                }
            });
            cleanIds.forEach((id) => {
                let item = this.msgQueue.get(id);
                if (item) {
                    this.handleFailMsg({
                        id,
                        url: "",
                        mode: RpcConfig_1.InteractiveMode.request,
                    }, {
                        code: RpcConfig_1.RpcResponseCode.timeout,
                        msg: `The message times out ${item.timeout} ms`,
                    });
                }
                this.msgQueue.delete(id);
            });
        }
        catch (e) {
            this.rpcLogger.error("rpc server checkMsgQueue error");
            this.rpcLogger.error(e);
        }
        finally {
            this.checkStatus = false;
        }
    }
    use(m) {
        this.middleware.push(m);
        this.setMiddleware();
    }
    getResponseMiddleware() {
        let onerror = this.app.getComponentByName(RpcMetaData_1.RpcMetaData.RPCErrorService);
        if (!!onerror) {
            return onerror.response();
        }
        return this.response();
    }
    setMiddleware() {
        let newList = [this.getResponseMiddleware()];
        let middlerList = this.app.getSetting(RpcMetaData_1.RpcMetaData.RPCMIDDLEWARE) || [];
        newList = [...newList, ...middlerList, ...this.middleware];
        const routerMiddleware = this.loadRoute();
        if (routerMiddleware) {
            newList.push(routerMiddleware);
        }
        this.composeMiddleware = (0, ComposeService_1.default)(newList);
    }
    async start() {
        //加载中间件
        this.setMiddleware();
        //这个步骤放在最后做
        let rpcConfig = this.app.getSetting(RpcMetaData_1.RpcMetaData.RpcConfig);
        if (!rpcConfig) {
            this.rpcLogger.warn(`This ${RpcMetaData_1.RpcMetaData.RpcConfig} was not found`);
            return;
        }
        let serverList = rpcConfig.list;
        if (serverList.length == 0) {
            this.rpcLogger.warn(`RPC Server list is empty`);
            return;
        }
        this.socketManager.bind(this);
        this.socketManager.start(serverList);
        this.rpcConfig.list = serverList;
        if (rpcConfig.retry) {
            Object.assign(this.rpcConfig.retry, rpcConfig.retry);
        }
        //开启定时检测
        this.checkMsgQueue(0, false);
    }
    async stop() {
        this.checkMsgQueue(0, true);
        this.failMsgQueue = [];
        this.msgQueue.clear();
        await this.socketManager.stop();
    }
};
__decorate([
    annotation_1.Autowired,
    __metadata("design:type", fastcar_core_1.FastCarApplication)
], RpcServer.prototype, "app", void 0);
__decorate([
    (0, annotation_1.Log)("rpc"),
    __metadata("design:type", fastcar_core_1.Logger)
], RpcServer.prototype, "rpcLogger", void 0);
__decorate([
    (0, fastcar_timer_1.ScheduledInterval)({
        initialDelay: 1000,
        fixedRate: 100, //大致100ms执行一次
    }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Boolean]),
    __metadata("design:returntype", Promise)
], RpcServer.prototype, "checkMsgQueue", null);
RpcServer = __decorate([
    (0, annotation_1.ApplicationStart)(fastcar_core_1.BootPriority.Lowest * 10, "start") //落后于koa执行
    ,
    (0, annotation_1.ApplicationStop)(fastcar_core_1.BootPriority.Base, "stop"),
    fastcar_timer_1.EnableScheduling,
    __metadata("design:paramtypes", [])
], RpcServer);
exports.default = RpcServer;
