"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SocketCodingDefault_1 = require("../../constant/SocketCodingDefault");
const CodeProtocolEnum_1 = require("../../types/CodeProtocolEnum");
//这边主要做对于各个集成的socket的一个约定
class SocketServer {
    constructor(config, manager) {
        this.id = config.id;
        this.sessions = new Map();
        this.manager = manager;
        this.state = false;
        this.config = config;
        switch (config.codeProtocol) {
            case CodeProtocolEnum_1.CodeProtocolEnum.PROTOBUF: {
                this.encode = SocketCodingDefault_1.EncodePBDefault;
                this.decode = SocketCodingDefault_1.DecodePBDefault;
                break;
            }
            case CodeProtocolEnum_1.CodeProtocolEnum.JSON:
            default: {
                this.encode = SocketCodingDefault_1.EncodeDefault;
                this.decode = SocketCodingDefault_1.DecodeDefault;
                break;
            }
        }
        if (config.encode) {
            this.encode = config.encode;
        }
        if (config.decode) {
            this.decode = config.decode;
        }
    }
    async sendMsgBySessionId(sessionId, msg) {
        return await this.sendMsg(sessionId, msg);
    }
    receiveMsg(sessionId, msg) {
        this.manager.handleMsg(sessionId, this.decode(msg));
    }
    connect(client, id, remoteAddress) {
        let session = {
            id: id,
            client,
            remoteAddress,
        };
        this.sessions.set(id, session);
        this.manager.connect(id);
    }
    //掉线处理
    disconnect(sessionId) {
        if (!this.sessions.has(sessionId)) {
            return false;
        }
        this.sessions.delete(sessionId);
        return true;
    }
    //主动掉线
    kickConnect(info) {
        let sessionId = info.sessionId;
        let session = this.getSession(sessionId);
        if (!session) {
            return false;
        }
        this.offline(session, info?.reason);
        this.disconnect(sessionId);
        return true;
    }
    //被动掉线
    dropConnect(sessionId, reason) {
        let session = this.getSession(sessionId);
        if (!session) {
            return false;
        }
        this.manager.disconnect(sessionId, reason, false);
        this.disconnect(sessionId);
        return true;
    }
    getSession(sessionId) {
        return this.sessions.get(sessionId);
    }
    getSessions() {
        return [...this.sessions.values()];
    }
    getConfig() {
        return this.config;
    }
}
exports.default = SocketServer;
