"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const grpc = require("@grpc/grpc-js");
const protoLoader = require("@grpc/proto-loader");
const SocketEnum_1 = require("../../../../constant/SocketEnum");
const SocketClient_1 = require("../../SocketClient");
const SocketEvents_1 = require("../../../../types/SocketEvents");
const ProtoBuffService_1 = require("../../../ProtoBuffService");
const SocketCodingDefault_1 = require("../../../../constant/SocketCodingDefault");
const fs = require("fs");
class GrpcClient extends SocketClient_1.SocketClient {
    constructor(config, manager) {
        super(config, manager);
        this.type = SocketEnum_1.SocketEnum.Grpc;
    }
    connect() {
        if (this.connected) {
            this.disconnect();
        }
        const PROTO_PATH = ProtoBuffService_1.default.getRouterRootPath();
        const packageDefinition = protoLoader.loadSync(PROTO_PATH, { keepCase: true, longs: String, enums: String, defaults: true, oneofs: true });
        const protoDescriptor = grpc.loadPackageDefinition(packageDefinition);
        const routerProto = protoDescriptor.router;
        let metadata = new grpc.Metadata();
        metadata.set("username", this.config.secure?.username || "");
        metadata.set("password", this.config.secure?.password || "");
        let ssl = this.config.ssl;
        let options = this.config.extra?.options || {};
        let instance = ssl ? grpc.credentials.createSsl(fs.readFileSync(ssl.ca), fs.readFileSync(ssl.key), fs.readFileSync(ssl.cert)) : grpc.ChannelCredentials.createInsecure();
        let client = new routerProto.Router(`${this.config.url}`, instance, options);
        const io = client.transferRoute(metadata);
        io.on("data", (response) => {
            this.receiveMsg(response);
        });
        io.on("end", () => {
            if (this.connected) {
                this.disconnect();
            }
        });
        this.forceConnect = false;
        this.io = io;
    }
    disconnect(reason) {
        if (this.connected) {
            this.connected = false;
            this.io.end();
        }
    }
    async sendMsg(msg) {
        if (!this.io || !this.connected) {
            return false;
        }
        return new Promise((resolve) => {
            try {
                this.io.write((0, SocketCodingDefault_1.EncodePBGrpcDefault)(msg));
                resolve(true);
            }
            catch (err) {
                if (err) {
                    this.logger.error("send msg fail");
                    this.logger.error(err);
                }
                resolve(!err);
            }
        });
    }
    receiveMsg(msg) {
        let result = (0, SocketCodingDefault_1.DecodePBGrpcDefault)(msg);
        if (!!result && result?.url == SocketEvents_1.SocketEvents.CONNECT_RECEIPT) {
            this.sessionId = result.data?.socketId;
            this.connected = true;
            return;
        }
        this.manager.handleMsg(result);
    }
    offline(reason) {
        if (this.io && this.connected) {
            this.disconnect(reason);
        }
    }
    close() {
        this.offline();
    }
}
exports.default = GrpcClient;
