"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const SocketServer_1 = require("../../SocketServer");
const grpc = require("@grpc/grpc-js");
const protoLoader = require("@grpc/proto-loader");
const RpcConfig_1 = require("../../../../types/RpcConfig");
const annotation_1 = require("fastcar-core/annotation");
const fastcar_core_1 = require("fastcar-core");
const SocketEvents_1 = require("../../../../types/SocketEvents");
const ProtoBuffService_1 = require("../../../ProtoBuffService");
const SocketCodingDefault_1 = require("../../../../constant/SocketCodingDefault");
class GrpcServer extends SocketServer_1.default {
    constructor(config, manager) {
        super(config, manager);
    }
    async listen() {
        const PROTO_PATH = ProtoBuffService_1.default.getRouterRootPath();
        const packageDefinition = protoLoader.loadSync(PROTO_PATH, { keepCase: true, longs: String, enums: String, defaults: true, oneofs: true });
        const protoDescriptor = grpc.loadPackageDefinition(packageDefinition);
        const routerProto = protoDescriptor.router;
        let server = new grpc.Server();
        server.addService(routerProto["Router"].service, {
            transferRoute: async (socket) => {
                //接收消息
                let metedataMap = socket.metadata.internalRepr;
                let session = this.manager.createSession(this.id);
                let username = metedataMap.get("username")[0];
                if (!username) {
                    username = "";
                }
                else {
                    username = username.toString();
                }
                let password = metedataMap.get("password")[0];
                if (!password) {
                    password = "";
                }
                else {
                    password = password.toString();
                }
                let flag = await this.manager.auth(username, password, session);
                if (!flag) {
                    this.manager.deleteSession(session.sessionId);
                    socket.close(3000, "auth fail");
                    return;
                }
                let socketId = session.sessionId;
                let remoteAddress = socket?.call?.stream?.session?.socket?.remoteAddress;
                this.connect(socket, socketId, remoteAddress || "0.0.0.0");
                socket.write((0, SocketCodingDefault_1.EncodePBGrpcDefault)({
                    url: SocketEvents_1.SocketEvents.CONNECT_RECEIPT,
                    mode: RpcConfig_1.InteractiveMode.request,
                    data: { socketId },
                }));
                socket.on("data", (request) => {
                    this.manager.handleMsg(socketId, (0, SocketCodingDefault_1.DecodePBGrpcDefault)(request));
                });
                socket.on("end", () => {
                    this.dropConnect(socketId, "client end");
                });
            },
        });
        let ssl = this.config.server.ssl;
        let { checkClientCertificate } = this.config.extra;
        let netServer = ssl
            ? grpc.ServerCredentials.createSsl(Buffer.from(this.app.getFileContent(ssl.ca)), [
                {
                    cert_chain: Buffer.from(this.app.getFileContent(ssl.cert)),
                    private_key: Buffer.from(this.app.getFileContent(ssl.key)),
                },
            ], checkClientCertificate)
            : grpc.ServerCredentials.createInsecure();
        server.bindAsync(`${this.config.server.hostname || "0.0.0.0"}:${this.config.server.port}`, netServer, () => {
            server.start();
            this.rpcLogger.debug(`grpc server started on ${this.config.server.port}`);
        });
        this.server = server;
    }
    async close() {
        if (!!this.server) {
            this.server.forceShutdown();
        }
    }
    async sendMsg(sessionId, msg) {
        let socket = this.getSession(sessionId);
        if (!socket) {
            return false;
        }
        socket.client.write((0, SocketCodingDefault_1.EncodePBGrpcDefault)(msg));
        return true;
    }
    offline(session, reason) {
        session.client.end();
    }
}
__decorate([
    (0, annotation_1.Log)("rpc"),
    __metadata("design:type", fastcar_core_1.Logger)
], GrpcServer.prototype, "rpcLogger", void 0);
__decorate([
    annotation_1.CallDependency,
    __metadata("design:type", fastcar_core_1.FastCarApplication)
], GrpcServer.prototype, "app", void 0);
exports.default = GrpcServer;
