"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SocketEnum_1 = require("../../../../constant/SocketEnum");
const SocketClient_1 = require("../../../socket/SocketClient");
const ioClient = require("socket.io-client");
const SocketEvents_1 = require("../../../../types/SocketEvents");
class IoSocketClient extends SocketClient_1.SocketClient {
    constructor(config, manager) {
        super(config, manager);
        config.extra = Object.assign({}, { forceNew: true, transports: ["websocket"] }, config.extra, {
            query: {
                username: config.secure?.username,
                password: config.secure?.password,
            },
        });
        this.type = SocketEnum_1.SocketEnum.SocketIO;
    }
    connect() {
        if (this.connected) {
            this.disconnect();
        }
        const io = ioClient.connect(this.config.url, this.config.extra);
        io.on(SocketEvents_1.SocketEvents.CONNECT_RECEIPT, (socketId) => {
            this.sessionId = socketId;
            this.connected = true;
        });
        io.on(SocketEvents_1.SocketEvents.DISCONNECT, () => {
            if (this.connected) {
                this.connected = false;
                // this.disconnect();
            }
        });
        io.on(SocketEvents_1.SocketEvents.MESSAGE, (msg) => {
            this.receiveMsg(msg);
        });
        this.forceConnect = false;
        this.io = io;
    }
    disconnect(reason) {
        this.connected = false;
        this.io.close();
        this.logger.warn(`client disconnect ${reason}`);
    }
    async sendMsg(msg) {
        if (!this.connected) {
            return false;
        }
        this.io.emit(SocketEvents_1.SocketEvents.MESSAGE, this.encode(msg));
        return true;
    }
    receiveMsg(msg) {
        //协议进行解析
        this.manager.handleMsg(this.decode(msg));
    }
    offline(reason) {
        if (this.io && this.connected) {
            this.disconnect(reason);
        }
    }
    close() {
        if (this.io) {
            this.disconnect();
        }
    }
}
exports.default = IoSocketClient;
