"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SocketServer_1 = require("../../SocketServer");
const socket_io_1 = require("socket.io");
const SocketEvents_1 = require("../../../../types/SocketEvents");
class IoSocketServer extends SocketServer_1.default {
    constructor(config, manager) {
        super(config, manager);
        this.sessionMap = new Map();
    }
    async listen() {
        let netServer = this.manager.createNetServer(this.config.server);
        this.server = new socket_io_1.Server(netServer, Object.assign({ transports: ["websocket"] }, this.config.extra));
        //预处理连接认证
        this.server.use(async (socket, next) => {
            let session = this.manager.createSession(this.id);
            let { username, password } = socket.handshake.query;
            if (!username) {
                username = "";
            }
            else {
                username = username.toString();
            }
            if (!password) {
                password = "";
            }
            else {
                password = password.toString();
            }
            let flag = await this.manager.auth(username, password, session);
            if (!flag) {
                this.manager.deleteSession(session.sessionId);
                return next(new Error(`socketio authentication  error`));
            }
            this.sessionMap.set(socket.id, session.sessionId);
            next();
        });
        this.server.on(SocketEvents_1.SocketEvents.CONNECT, (socket) => {
            let socketId = this.sessionMap.get(socket.id) || "";
            if (!socketId) {
                socket.disconnect(true);
                return;
            }
            this.connect(socket, socketId, socket.handshake.address);
            //给客户端发送消息
            socket.emit(SocketEvents_1.SocketEvents.CONNECT_RECEIPT, socketId);
            //监听客户端断开连接的消息
            socket.on(SocketEvents_1.SocketEvents.DISCONNECT, (reason) => {
                this.dropConnect(socketId, reason);
            });
            //监听消息
            socket.on(SocketEvents_1.SocketEvents.MESSAGE, (msg) => {
                this.receiveMsg(socketId, msg);
            });
        });
    }
    async close() {
        await new Promise((resolve) => {
            this.server.close((err) => {
                if (err) {
                    this.manager.getLogger().error(err);
                }
                resolve("OK");
            });
        });
    }
    getSession(sessionId) {
        return this.sessions.get(sessionId);
    }
    async sendMsg(sessionId, msg) {
        let socket = this.getSession(sessionId);
        if (!socket) {
            return false;
        }
        //根据协议转成想要的服务
        socket.client.emit(SocketEvents_1.SocketEvents.MESSAGE, this.encode(msg));
        return true;
    }
    offline(session, reason) {
        session.client.disconnect(true);
    }
}
exports.default = IoSocketServer;
