"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mqtt = require("mqtt");
const SocketEnum_1 = require("../../../../constant/SocketEnum");
const SocketClient_1 = require("../../SocketClient");
const SocketEvents_1 = require("../../../../types/SocketEvents");
class MqttSocketClient extends SocketClient_1.SocketClient {
    constructor(config, manager) {
        super(config, manager);
        config.extra = Object.assign({}, config.extra);
        this.type = SocketEnum_1.SocketEnum.MQTT;
    }
    connect() {
        //默认没有用户名和密码
        if (!this.config.secure) {
            this.config.secure = {
                username: "",
                password: "",
            };
        }
        const io = mqtt.connect(this.config.url, Object.assign({ reconnecting: true }, this.config.extra, this.config.secure));
        io.on(SocketEvents_1.SocketEvents.CONNECT, () => {
            this.connected = true;
        });
        io.on(SocketEvents_1.SocketEvents.MESSAGE, (topic, message) => {
            if (topic == SocketEvents_1.SocketEvents.CONNECT_RECEIPT) {
                this.connected = true;
                this.sessionId = message.toString();
            }
            else {
                this.receiveMsg(message);
            }
        });
        io.on("error", (error) => {
            // this.disconnect(packet.reasonCode?.toString());
            let code = error.code;
            if (this.connected) {
                if (code == "ECONNRESET" || code == "ECONNREFUSED") {
                    this.connected = false;
                }
            }
        });
        this.io = io;
        this.forceConnect = false;
    }
    disconnect(reason) {
        this.connected = false;
        this.io.end();
        this.logger.warn(`client disconnect ${reason}`);
    }
    async sendMsg(msg) {
        if (!this.connected) {
            return false;
        }
        return new Promise((resolve) => {
            this.io.publish(SocketEvents_1.SocketEvents.MESSAGE, this.encode(msg), (error) => {
                resolve(!error);
            });
        });
    }
    receiveMsg(msg) {
        this.manager.handleMsg(this.decode(msg));
    }
    offline(reason) {
        if (this.io && this.connected) {
            this.disconnect(reason);
        }
    }
    close() {
        if (this.io) {
            this.disconnect();
        }
    }
}
exports.default = MqttSocketClient;
