"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SocketServer_1 = require("../../SocketServer");
const SocketEvents_1 = require("../../../../types/SocketEvents");
const aedes_1 = require("aedes");
const fastcar_server_1 = require("fastcar-server");
const net = require("net");
class MqttSocketServer extends SocketServer_1.default {
    constructor(config, manager) {
        super(config, manager);
        if (!this.config.secure) {
            this.config.secure = {
                username: "",
                password: "",
            };
        }
        this.sessionMap = new Map();
    }
    async listen() {
        this.server = (0, aedes_1.Server)({
            id: this.config.id,
            authenticate: (client, username, password, callback) => {
                //创建一个会话
                let session = this.manager.createSession(this.id);
                this.manager.auth(username, password.toString(), session).then((res) => {
                    if (!res) {
                        this.manager.deleteSession(session.sessionId);
                    }
                    this.sessionMap.set(client.id, session.sessionId);
                    callback(null, res);
                });
            },
            authorizePublish: (client, packet) => {
                let sessionId = this.sessionMap.get(client.id);
                if (sessionId) {
                    this.receiveMsg(sessionId, packet.payload);
                }
            },
        });
        this.server.on("client", (socket) => {
            let remoteAddress = "0.0.0.0";
            if (socket.conn instanceof net.Socket) {
                remoteAddress = socket.conn.remoteAddress || "0.0.0.0";
            }
            let socketId = this.sessionMap.get(socket.id);
            if (!socketId) {
                return;
            }
            //给客户端发送消息
            this.connect(socket, socketId, remoteAddress);
            socket.publish({
                topic: SocketEvents_1.SocketEvents.CONNECT_RECEIPT,
                payload: socketId,
                cmd: "publish",
                qos: 0,
                retain: true,
                dup: false,
            }, () => { });
        });
        //监听客户端断开连接的消息
        this.server.on("clientDisconnect", (client) => {
            let sessionId = this.sessionMap.get(client.id);
            if (sessionId) {
                this.dropConnect(sessionId, "client disconnect");
                this.sessionMap.delete(client.id);
            }
        });
        if (this.config.server.protocol == fastcar_server_1.Protocol.http || this.config.server.protocol == fastcar_server_1.Protocol.https) {
            let httpServer = this.manager.createNetServer(this.config.server);
            const ws = require("websocket-stream");
            ws.createServer({ server: httpServer }, this.server.handle);
        }
        else {
            this.manager.createNetServer(this.config.server, this.server.handle);
        }
    }
    async close() {
        await new Promise((resolve) => {
            this.server.close(() => {
                resolve("OK");
            });
        });
    }
    getSession(sessionId) {
        return this.sessions.get(sessionId);
    }
    async sendMsg(sessionId, msg) {
        let socket = this.getSession(sessionId);
        if (!socket) {
            return false;
        }
        return new Promise((resolve) => {
            //根据协议转成想要的服务
            let content = this.encode(msg);
            socket.client.publish({
                topic: "message",
                payload: content,
                cmd: "publish",
                qos: 0,
                retain: false,
                dup: false,
            }, (err) => {
                resolve(!err);
            });
        });
    }
    offline(session, reason) {
        session.client.close();
    }
}
exports.default = MqttSocketServer;
