"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SocketEnum_1 = require("../../../../constant/SocketEnum");
const SocketClient_1 = require("../../SocketClient");
const ws_1 = require("ws");
const SocketEvents_1 = require("../../../../types/SocketEvents");
class WsSocketClient extends SocketClient_1.SocketClient {
    constructor(config, manager) {
        super(config, manager);
        this.type = SocketEnum_1.SocketEnum.WS;
    }
    connect() {
        if (this.connected) {
            this.disconnect();
        }
        const io = new ws_1.WebSocket(this.config.url, Object.assign({}, this.config.extra, {
            headers: {
                username: this.config.secure?.username || "",
                password: this.config.secure?.password || "",
            },
        }));
        io.on(SocketEvents_1.SocketEvents.CLOSE, () => {
            if (this.connected) {
                this.connected = false;
            }
        });
        io.on(SocketEvents_1.SocketEvents.MESSAGE, (msg) => {
            this.receiveMsg(msg);
        });
        this.forceConnect = false;
        this.io = io;
    }
    disconnect(reason) {
        this.connected = false;
        this.io.close();
        this.logger.warn(`client disconnect ${reason}`);
    }
    async sendMsg(msg) {
        if (!this.io || !this.connected) {
            return false;
        }
        return new Promise((resolve) => {
            this.io.send(this.encode(msg), (err) => {
                if (err) {
                    this.logger.error("send msg fail");
                    this.logger.error(err);
                }
                resolve(!err);
            });
        });
    }
    receiveMsg(msg) {
        let result = this.decode(msg);
        if (!!result && result?.url == SocketEvents_1.SocketEvents.CONNECT_RECEIPT) {
            this.sessionId = result.data?.socketId;
            this.connected = true;
            return;
        }
        this.manager.handleMsg(result);
    }
    offline(reason) {
        if (this.io && this.connected) {
            this.disconnect(reason);
        }
    }
    close() {
        this.offline();
    }
}
exports.default = WsSocketClient;
