"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SocketServer_1 = require("../../SocketServer");
const ws_1 = require("ws");
const SocketEvents_1 = require("../../../../types/SocketEvents");
const RpcConfig_1 = require("../../../../types/RpcConfig");
class WsSocketServer extends SocketServer_1.default {
    async listen() {
        let netServer = this.manager.createNetServer(this.config.server);
        this.server = new ws_1.WebSocketServer(Object.assign({}, { server: netServer }, this.config.extra));
        this.server.on("connection", async (socket, request) => {
            //进行校验
            let session = this.manager.createSession(this.id);
            let { username, password } = request.headers;
            if (!username) {
                username = "";
            }
            else {
                username = username.toString();
            }
            if (!password) {
                password = "";
            }
            else {
                password = password.toString();
            }
            let flag = await this.manager.auth(username, password, session);
            if (!flag) {
                this.manager.deleteSession(session.sessionId);
                socket.close(3000, "auth fail");
                return;
            }
            let socketId = session.sessionId;
            this.connect(socket, socketId, request.socket.remoteAddress || "0.0.0.0");
            socket.send(this.encode({
                url: SocketEvents_1.SocketEvents.CONNECT_RECEIPT,
                mode: RpcConfig_1.InteractiveMode.request,
                data: { socketId },
            }));
            socket.on(SocketEvents_1.SocketEvents.MESSAGE, (data) => {
                this.receiveMsg(socketId, data);
            });
            socket.on("close", (code, reason) => {
                this.dropConnect(socketId, code + " " + reason.toString());
            });
        });
    }
    async close() {
        await new Promise((resolve) => {
            this.server.close((err) => {
                if (err) {
                    this.manager.getLogger().error(err);
                }
                resolve("OK");
            });
        });
    }
    async sendMsg(sessionId, msg) {
        let socket = this.getSession(sessionId);
        if (!socket) {
            return false;
        }
        console.debug("服务端发送消息模式", msg.mode);
        socket.client.send(this.encode(msg));
        return true;
    }
    getSession(sessionId) {
        return this.sessions.get(sessionId);
    }
    offline(session, reason) {
        session.client.close(3001, "Server forced shutdown");
    }
}
exports.default = WsSocketServer;
